#define FASTADC 1
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi

#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#include <Wire.h>
#include "Leds.h"
#include "Utils.h"
#include "Tlc5940.h"

	
#define VREF     5.0  // analog reference
#define LED      64   // led number
#define MAX      2     
#define DELTA    21   

Leds morse(7);
Utils util;

boolean update            = false;
boolean contact           = false;

byte buffer[5] ;
int  p             = 0;
int  delayTime     = 2;
int  lum           = MAX;
int  debug         = 1; // debug = 0 disabled - debug = 1 enabled
long oldTime       = 0;

void setup()
{ 
  Tlc.init();

  Serial.begin(115200);
  morse.led();
  
  establishContact();
}

void establishContact() {
  while (contact==false) {
    Serial.println('B');
    delay(200);
    
    if (Serial.read()=='K'){//K
      contact=true; 
      morse.off(); // offLed();
      if(debug!=0) Serial.println('K');
     
    }      
  }
}
 
void loop()
{
  if (Serial.available() >0)
  {
    
    morse.led();
    
    byte inByte = Serial.read();
    
    if (inByte == '#') // # PP RR led number and value 
    {
      while (p<4)
      {
        buffer[p] = Serial.read();
        p++;
      }
      byte pp    = util.hex2Decimal(buffer[1])+util.hex2Decimal(buffer[0])*16;
      byte led   = util.hex2Decimal(buffer[3])+util.hex2Decimal(buffer[2])*16;
      if(debug!=0){
        Serial.print("# ");
        Serial.print(pp);
        Serial.print(" ");
        Serial.println(led);
      }
        
      handleLed(pp, led);
      update=true;
    }

    else if (inByte == 'L') // L XX intensity
    {
      while (p<2)
      {
        buffer[p] = Serial.read();
        p++;        
      }
      lum = util.hex2Decimal(buffer[1])+util.hex2Decimal(buffer[0])*16;
      if(debug!=0){
        Serial.print("L ");
        Serial.println(lum);
      }
    }

    else if (inByte == 'T') // T XX delay time
    {
      while (p<2)
      {
        buffer[p] = Serial.read();
        p++;        
      }
      delayTime = util.hex2Decimal(buffer[1])+util.hex2Decimal(buffer[0])*16;
      if(debug!=0){
        Serial.print("T ");
        Serial.println(delayTime);
      }
    }
    else if (inByte == 'S') // switch off all leds
    {
      Tlc.clear(); Tlc.update();
      if(debug!=0){
        Serial.println("S ");
      }
    } 
    else if (inByte =='X') // X XX active (1) - no active (0) debug 
    {
      while (p<2)
      {
        buffer[p] = Serial.read();
        p++;        
      }
      debug = util.hex2Decimal(buffer[1])+util.hex2Decimal(buffer[0])*16;
      if(debug!=0){
        Serial.print("X ");
        Serial.println(debug);
      }
    } 
    morse.off(); 
  }
  p=0;

  if(update & millis()-oldTime>DELTA){
    Tlc.update(); 
    update=false;
    oldTime=millis();
  }
  delay(delayTime); 
}

/****************************************************** TLC LED RGB */
void handleLed(int pin, int led)
{
  Tlc.set(pin+0, led*lum);
}
