
#define FASTADC 1
#ifndef cbi
#define cbi(sfr, bit) (_SFR_BYTE(sfr) &= ~_BV(bit))
#endif
#ifndef sbi
#define sbi(sfr, bit) (_SFR_BYTE(sfr) |= _BV(bit))
#endif

#include <Wire.h>
#include "Leds.h"
#include "Utils.h"
#include "Tlc5940.h"
#include "tlc_servos.h"


	
#define VREF     5.0  // analog reference
#define SERVO    64   // servo number
#define DELTA    61   

Leds morse(7);
Utils util;

boolean update            = false;
boolean contact           = false;

char buffer[5] ;
int  p             = 0;
int  delayTime     = 2;
int  debug         = 1; // debug = 0 disabled - debug = 1 enabled
long oldTime       = 0;

void setup()
{ 
  tlc_initServos(); 

  Serial.begin(9600);
  morse.led();
 
  establishContact();
}

void establishContact() {
  while (contact==false) {
    Serial.println('S');
    delay(200);
    
    if (Serial.read()=='K'){//K
      contact=true; 
      morse.off(); // offLed();
      if(debug!=0) Serial.println('K');
     
    }      
  }
}
 
void loop()
{
  if (Serial.available() >0)
  {
    
    morse.led();
    
    char inByte = Serial.read();
    
    if (inByte =='#') // # PP RR servo number and value 
    {
     
      while (p<4)
      {
        buffer[p] = Serial.read();
        Serial.println(buffer[p]);
        p++;
      }
      byte pp    = util.hex2Decimal(buffer[1])+util.hex2Decimal(buffer[0])*16;
      byte angle = util.hex2Decimal(buffer[3])+util.hex2Decimal(buffer[2])*16;
      if(debug!=0){
        Serial.print("# ");
        Serial.print(pp);
        Serial.print(" ");
        Serial.println(angle);
      }
      if (angle>4 && angle<176){ 
        handleServo(pp, angle);
        update=true;
      }
    }

    else if (inByte =='T') // T XX delay time
    {
      while (p<2)
      {
        buffer[p] = Serial.read();
        p++;        
      }
      delayTime = util.hex2Decimal(buffer[1])+util.hex2Decimal(buffer[0])*16;
      if(debug!=0){
        Serial.print("T ");
        Serial.println(delayTime);
      }
    }
    else if (inByte =='S') // S reset servo position
    {
      for(int i=0; i<SERVO; i++)
      {
        handleServo(i, 0);
      }
      Tlc.update();
      if(debug!=0){
        Serial.println("S ");
      }
    } 
    else if (inByte =='X') // X XX activate (1) - not active (0) debug 
    {
      while (p<2)
      {
        buffer[p] = Serial.read();
        p++;        
      }
      debug = util.hex2Decimal(buffer[1])+util.hex2Decimal(buffer[0])*16;      
      Serial.print("X ");
      Serial.println(debug);
      
    } 
    morse.off(); 
  }
  p=0;
  
  if(update & millis()-oldTime>DELTA){
    Tlc.update(); 
    update=false;
    oldTime=millis();
  }
  delay(delayTime); 
}

/****************************************************** TLC SERVOS */
void handleServo(int pin, int angle)
{
  if(debug!=0){
        Serial.println("moving servo ");
        Serial.print(pin);
        Serial.print(" ");
        Serial.print(angle);
      } 
  tlc_setServo(pin+0, angle);
}
